import 'package:flutter/material.dart';
import 'package:osam/osam.dart';
import 'package:provider/provider.dart';
import 'package:provider/single_child_widget.dart';

part '../../presentation/presenter.dart';
part '../../presentation/usecase_provider.dart';

class UseCaseNotFoundException implements Exception {
  final Type useCaseType;

  UseCaseNotFoundException(this.useCaseType);
}

class UseCaseContainer {
  final List<Object> _useCases;
  final _useCasesMap = <Type, Object>{};

  U useCase<U extends Object>() {
    final useCase = _useCasesMap[U];
    if (useCase == null) {
      throw UseCaseNotFoundException(U);
    } else
      return useCase;
  }

  UseCaseContainer(this._useCases) {
    if (_useCases?.isNotEmpty ?? false) {
      for (final useCase in _useCases) {
        _useCasesMap[useCase.runtimeType] = useCase;
      }
    }
  }

  void updateUseCases(List<Object> newUseCases) =>
      newUseCases.forEach((useCase) => _useCasesMap[useCase.runtimeType] = useCase);
}
