import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:osam/osam.dart';
import 'package:osam/util/value_stream.dart';

part 'base_value.g.dart';

typedef Change<T> = void Function(T value);

@HiveType(typeId: 223)
// ignore: must_be_immutable
class BaseValue<T extends Object> extends Equatable {
  int _lastKnownHashCode;
  @HiveField(0)
  T _value;
  final _valueBroadcaster = StreamController<T>.broadcast();

  BaseValue(this._value);

  ValueStream<T> get valueStream => ValueStream(_valueBroadcaster.stream, _value);

  T get value => _value;

  void _update() {
    if (_lastKnownHashCode != hashCode) {
      _valueBroadcaster.sink.add(_value);
      _lastKnownHashCode = hashCode;
    }
  }

  void throwError(Object error) => _valueBroadcaster.sink.addError(error);

  set value(T value) {
    _value = value;
    _update();
  }

  //useful for iterables (add, remove and so on)...
  void change(Change<T> change) {
    change(_value);
    _update();
  }

  void clear(T newValue) => _value = newValue;

  void destruct() => _valueBroadcaster.close();

  @override
  List<T> get props => [_value];
}

mixin BaseValueMixin<T> on Equatable {
  int _lastKnownHashCode = -1;

  final _valueBroadcaster = StreamController<T>.broadcast();

  ValueStream<T> get valueStream => ValueStream(_valueBroadcaster.stream, this as T);

  void _update() {
    if (_lastKnownHashCode != hashCode) {
      _valueBroadcaster.sink.add(this as T);
      _lastKnownHashCode = hashCode;
    }
  }

  //useful for iterables (add, remove and so on)...
  void change(Change<T> change) {
    change(this as T);
    _update();
  }

  void destruct() => _valueBroadcaster.close();

  @override
  List<T> get props;
}
