import 'dart:async';

import '../osam.dart';

mixin NavigationPresenter<US> on Presenter<US> {
  NavigationState get navigationState;

  StreamSubscription<List<String>> routesSub;

  @override
  void init() {
    routesSub = navigationState.routesStream.listen((routes) {});
  }

  ValueStream<List<String>> get routesStream => navigationState.routesStream;

  void pop() => navigationState.pop();

  void push(String route) => navigationState.push(route);

  void pushAndRemoveUntil(String route) => navigationState.pushAndRemoveUntil(route);

  void pushReplacement(String route) => navigationState.pushReplacement(route);

  @override
  void dispose() {
    routesSub.cancel();
    super.dispose();
  }
}

mixin TabNavigationPresenter<US> on Presenter<US> {
  TabNavigationState get tabNavigationState;

  void changeTab(String tab) => tabNavigationState.update((state) => state.tab = tab);

  ValueStream<String> get tabStream => tabNavigationState.propertyStream((state) => state.tab);
}
