import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:osam/util/value_stream.dart';

typedef SnapshotBuilder<T> = Widget Function(BuildContext context, AsyncSnapshot<T> snapshot);
typedef ErrorBuilder = Widget Function(BuildContext context, Object error);

class OsamBuilder<T> extends StatelessWidget {
  final ValueStream<T> stream;
  final SnapshotBuilder<T> builder;
  final ErrorBuilder errorBuilder;

  const OsamBuilder({Key key, @required this.stream, @required this.builder, this.errorBuilder})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<T>(
        key: ValueKey(key.toString() + 'st_B'),
        stream: stream.stream,
        initialData: stream.value,
        builder: (ctx, snapshot) =>
            snapshot.hasError ? errorBuilder(ctx, snapshot.error) : builder(ctx, snapshot));
  }
}

class OsamValueBuilder<T> extends StatelessWidget {
  final ValueListenable<T> value;

  const OsamValueBuilder({Key key, this.value}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return ValueListenableBuilder<T>(
      valueListenable: value,
      builder: (ctx, value, widget) {
        return Container();
      },
    );
  }
}
